from django.test import TestCase

from rest_framework.reverse import reverse
from rest_framework.test import APIClient

from ovp_uploads.models import UploadedImage
from ovp_users.models import User

from uuid import UUID

from PIL import Image

from tempfile import NamedTemporaryFile


def is_valid_uuid(uuid_to_test, version=4):
  try:
    uuid_obj = UUID(uuid_to_test, version=version)
  except: # pragma: no cover
    return False

  return str(uuid_obj) == uuid_to_test

class UploadedImageModelTestCase(TestCase):
  def test_str_return_uuid(self):
    """Assert that image model __str__ method returns uuid"""
    img = UploadedImage()
    img.save()

    uuid = img.__str__()

    self.assertTrue(is_valid_uuid(uuid))

  def test_image_urls(self):
    """Assert that image object returns url"""
    user = User.objects.create_user('test_image_urls@test.com', 'validpassword')

    client = APIClient()
    client.force_authenticate(user=user)

    image = Image.new('RGB', (100, 100))
    tmp_file = NamedTemporaryFile()
    image.save(tmp_file, format="JPEG")

    tmp_file.seek(0) # otherwise we start reading at the end

    data = {
      'image': tmp_file
    }

    response = client.post(reverse('upload-images-list'), data, format="multipart")

    self.assertTrue(response.status_code == 201)

    img_id = response.data['id']
    img = UploadedImage.objects.get(pk=img_id)

    self.assertTrue(img.get_image_url())
    self.assertTrue(img.get_image_small_url())
    self.assertTrue(img.get_image_medium_url())
    self.assertTrue(img.get_image_large_url())



from django.conf.urls import url, include
from rest_framework import routers

from ovp_uploads import views

router = routers.DefaultRouter()
router.register(r'uploads/images', views.UploadedImageViewSet, 'upload-images')

urlpatterns = [
  url(r'^', include(router.urls)),
]



from django.test import TestCase

from rest_framework.reverse import reverse
from rest_framework.test import APIClient

from django.http.request import QueryDict

from ovp_users.models import User

from PIL import Image

from tempfile import NamedTemporaryFile

import json

class UploadedImageViewSetTestCase(TestCase):
  def test_cant_upload_unauthenticated(self):
    """Assert that it's not possible to upload while unauthenticated without custom header"""
    client = APIClient()
    response = client.post(reverse('upload-images-list'), {}, format="multipart")

    self.assertTrue(response.status_code == 401)

  def test_cant_upload_no_image(self):
    """Assert that error is raised on no image"""
    user = User.objects.create_user('test_cant_upload_no_image@test.com', 'validpassword')

    client = APIClient()
    client.force_authenticate(user=user)
    response = client.post(reverse('upload-images-list'), {}, format="multipart")

    self.assertTrue(response.status_code == 400)
    self.assertTrue(response.data['image'][0] == 'No file was submitted.')

  def test_can_upload_authenticated(self):
    """Assert that it's possible to upload while authenticated"""
    user = User.objects.create_user('test_can_upload@test.com', 'validpassword')

    client = APIClient()
    client.force_authenticate(user=user)

    image = Image.new('RGB', (100, 100))
    tmp_file = NamedTemporaryFile()
    image.save(tmp_file, format="JPEG")

    tmp_file.seek(0) # otherwise we start reading at the end

    data = {
      'image': tmp_file
    }

    response = client.post(reverse('upload-images-list'), data, format="multipart")

    self.assertTrue(response.status_code == 201)

  def test_can_upload_unauthenticated_with_header(self):
    """Assert that it's possible to upload while unauthenticated if sent correct header"""
    client = APIClient()

    image = Image.new('RGB', (100, 100))
    tmp_file = NamedTemporaryFile()
    image.save(tmp_file, format="JPEG")

    tmp_file.seek(0) # otherwise we start reading at the end

    data = {
      'image': tmp_file
    }

    response = client.post(reverse('upload-images-list'), data, format="multipart", HTTP_X_UNAUTHENTICATED_UPLOAD=True)

    data = json.loads(response.content.decode('utf-8'))

    self.assertTrue(response.status_code == 201)



from ovp_uploads.models import UploadedImage

from rest_framework import serializers

def build_absolute_uri(req, image):
  return self.context['request'].build_absolute_uri(image.url) if image else None


class UploadedImageSerializer(serializers.ModelSerializer):
  image_url = serializers.SerializerMethodField()
  image_small_url = serializers.SerializerMethodField()
  image_medium_url = serializers.SerializerMethodField()
  image_large_url = serializers.SerializerMethodField()

  class Meta:
    model = UploadedImage
    fields = ('id', 'user', 'image', 'image_url', 'image_small_url', 'image_medium_url', 'image_large_url')
    read_only_fields = ('image_small', 'image_medium', 'image_large')
    extra_kwargs = {'image': {'write_only': True}}

  def get_image_url(self, obj):
    return build_absolute_uri(self.context['request'], obj.image)

  def get_image_small_url(self, obj):
    return build_absolute_uri(self.context['request'], obj.image_small)

  def get_image_medium_url(self, obj):
    return build_absolute_uri(self.context['request'], obj.image_medium)

  def get_image_large_url(self, obj):
    return build_absolute_uri(self.context['request'], obj.image_large)



import copy

from ovp_uploads.models import UploadedImage
from ovp_uploads.serializers import UploadedImageSerializer

from rest_framework import mixins
from rest_framework import viewsets
from rest_framework import response
from rest_framework import status

class UploadedImageViewSet(mixins.CreateModelMixin, viewsets.GenericViewSet):
  queryset = UploadedImage.objects.all()
  serializer_class = UploadedImageSerializer

  def create(self, request, *args, **kwargs):
    # Create mutable copy of request data
    data = copy.deepcopy(request.data)
    try:
      data._mutable=True
    except AttributeError: #pragma: no cover
      pass

    data.pop('user', None)
    upload_header = request.META.get('HTTP_X_UNAUTHENTICATED_UPLOAD', None)
    is_authenticated = request.user.is_authenticated()

    if is_authenticated or upload_header:
      if upload_header:
        data['user'] = None

      if is_authenticated:
        data['user'] = request.user.id

      serializer = self.get_serializer(data=data)

      if serializer.is_valid():
        self.object = serializer.save()
        headers = self.get_success_headers(serializer.data)

        return response.Response(serializer.data, status=status.HTTP_201_CREATED, headers=headers)
      return response.Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)
    return response.Response(status=status.HTTP_401_UNAUTHORIZED)



import uuid

from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.utils import timezone
from django.utils.deconstruct import deconstructible

from django_resized import ResizedImageField

from ovp_users.models import User


@deconstructible
class ImageName(object):
    def __init__(self, sub_path=""):
        self.path = sub_path

    def __call__(self, instance, filename):
        ext = filename.split('.')[-1]
        filename = "%s.%s" % (instance.uuid, ext)
        return 'user-uploaded/images%s/%s' % (self.path, filename)

image = ImageName()
image_small = ImageName("-small")
image_medium = ImageName("-medium")
image_large = ImageName("-large")

class UploadedImage(models.Model):
  def save(self, *args, **kwargs):
    if not self.pk:
      self.uuid = str(uuid.uuid4())
    # If _committed == False, it means the image is not uploaded to s3 yet
    # (will be uploaded on super.save()). This means the image is being updated
    # So we update other images accordingly
    if not self.image._committed:
      self.image_small = self.image._file # ._file because we need the InMemoryUploadedFile instance
      self.image_medium = self.image._file
      self.image_large = self.image._file

    self.modified_date = timezone.now()

    return super(UploadedImage, self).save(*args, **kwargs)

  def __str__(self):
    return self.uuid

  class Meta:
    app_label = 'ovp_uploads'
    verbose_name = _('uploaded image')
    verbose_name_plural = _('uploaded images')

  user = models.ForeignKey('ovp_users.User', default=None, null=True, blank=True)
  image = models.ImageField(_('Image 350x260'), upload_to=image)
  image_small = ResizedImageField(size=[350, 260], upload_to=image_small, blank=True, null=True, default=None)
  image_medium = ResizedImageField(size=[420, 312], upload_to=image_medium, blank=True, null=True, default=None)
  image_large = ResizedImageField(size=[1260, 936], upload_to=image_large, blank=True, null=True, default=None)
  created_date = models.DateTimeField(auto_now_add=True)
  modified_date = models.DateTimeField(auto_now=True)
  uuid = models.CharField('UUID', max_length=36, default=None, null=False, blank=True)

